# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class ValueType(str, Enum):
    """
    A list of types, familiar to the Ui, that define the expected input or output types on a piece of data
    """

    """
    allowed enum values
    """
    STRING = 'String'
    INT = 'Int'
    DECIMAL = 'Decimal'
    DATETIME = 'DateTime'
    BOOLEAN = 'Boolean'
    MAP = 'Map'
    LIST = 'List'
    PROPERTYARRAY = 'PropertyArray'
    PERCENTAGE = 'Percentage'
    CODE = 'Code'
    ID = 'Id'
    URI = 'Uri'
    CURRENCYANDAMOUNT = 'CurrencyAndAmount'
    TRADEPRICE = 'TradePrice'
    CURRENCY = 'Currency'
    METRICVALUE = 'MetricValue'
    RESOURCEID = 'ResourceId'
    RESULTVALUE = 'ResultValue'
    CUTLOCALTIME = 'CutLocalTime'
    DATEORCUTLABEL = 'DateOrCutLabel'
    UNINDEXEDTEXT = 'UnindexedText'

    @classmethod
    def from_json(cls, json_str: str) -> ValueType:
        """Create an instance of ValueType from a JSON string"""
        return ValueType(json.loads(json_str))
